<?php
/**
 * TMS Mailer.
 *
 * @package   TMS_Mailer
 * @author    Spyder Trap
 * @license   GPL-2.0+
 * @link      http://spydertrap.com
 * @copyright 2013 Spyder Trap
 */

/**
 * Plugin class. This class should ideally be used to work with the
 * administrative side of the WordPress site.
 *
 * @package TMS_Mailer_Admin
 * @author  Spyder Trap <admin@spydertrap.com>
 */
class TMS_Mailer_Admin {

	/**
	 * Instance of this class.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	protected static $instance = null;

	/**
	 * Slug of the plugin screen.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected $plugin_screen_hook_suffix = null;

	/**
	 * Initialize the plugin by loading admin scripts & styles and adding a
	 * settings page and menu.
	 *
	 * @since 1.0.0
	 */
	private function __construct() {

		/*
		 * Call $plugin_slug from public plugin class.
		 */
		$plugin = TMS_Mailer::get_instance();
		$this->plugin_slug = $plugin->get_plugin_slug();
		$this->is_tms_enabled = $plugin->is_tms_enabled();
		$this->is_cron_enabled = $plugin->is_cron_enabled();

		// Load admin style sheet and JavaScript.
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_styles' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

		// Add the options page and menu item.
		add_action( 'admin_menu', array( $this, 'add_plugin_admin_menu' ) );

		// Add an action link pointing to the options page.
		$plugin_basename = plugin_basename( plugin_dir_path( __DIR__ ) . $this->plugin_slug . '.php' );
		add_filter( 'plugin_action_links_' . $plugin_basename, array( $this, 'add_action_links' ) );

		// Register all TMS mailer settings
		add_action( 'admin_init', array( $this, 'tms_mailer_register_settings' ) );

		// Add the reports page and menu item.
		add_action( 'admin_menu', array( $this, 'add_reports_admin_menu' ) );

	}

	/**
	 * Return an instance of this class.
	 *
	 * @since 1.0.0
	 * @return object    A single instance of this class.
	 */
	public static function get_instance() {

		// If the single instance hasn't been set, set it now.
		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}

	/**
	 * Register and enqueue admin-specific style sheet.
	 *
	 * @since 1.0.0
	 * @return null    Return early if no settings page is registered.
	 */
	public function enqueue_admin_styles() {

		if ( ! isset( $this->plugin_screen_hook_suffix ) ) {
			return;
		}

		$screen = get_current_screen();
		if ( $this->plugin_screen_hook_suffix == $screen->id ) {
			wp_enqueue_style( $this->plugin_slug .'-admin-styles', plugins_url( 'assets/css/admin.css', __FILE__ ), array(), TMS_Mailer::VERSION );
		}

	}

	/**
	 * Register and enqueue admin-specific JavaScript.
	 *
	 * @since 1.0.0
	 * @return null    Return early if no settings page is registered.
	 */
	public function enqueue_admin_scripts() {

		if ( ! isset( $this->plugin_screen_hook_suffix ) ) {
			return;
		}

		$screen = get_current_screen();
		if ( $this->plugin_screen_hook_suffix == $screen->id ) {
			wp_enqueue_script( $this->plugin_slug . '-admin-script', plugins_url( 'assets/js/admin.js', __FILE__ ), array( 'jquery' ), TMS_Mailer::VERSION );
		}

	}

	/**
	 * Register the administration menu for this plugin into the WordPress Dashboard menu.
	 *
	 * Register a load-(page) hook for the settings page.
	 * src: http://codex.wordpress.org/Plugin_API/Action_Reference/load-(page)
	 *
	 * @since 1.0.0
	 */
	public function add_plugin_admin_menu() {

		/*
		 * Add a settings page for this plugin to the Settings menu.
		 */
		$this->plugin_screen_hook_suffix = add_options_page(
			__( 'GovDelivery TMS Mailer Settings', $this->plugin_slug ),
			__( 'GD TMS Mailer', $this->plugin_slug ),
			'manage_options',
			$this->plugin_slug,
			array( $this, 'display_plugin_admin_page' )
		);

		/*
		 * Register a hook load-(page).
		 * Run/stop cron job after settings were saved.
		 */
		add_action( 'load-'.$this->plugin_screen_hook_suffix, array( $this, 'run_cron_job' ) );

		/*
		 * Register a hook load-(page).
		 * Send test email right from Settings page.
		 */
		add_action( 'load-'.$this->plugin_screen_hook_suffix, array( $this, 'send_test_email' ) );

	}

	/**
	 * Render the settings page for this plugin.
	 *
	 * @since 1.0.0
	 */
	public function display_plugin_admin_page() {
		include_once( 'views/admin.php' );
	}

	/**
	 * Add settings action link to the plugins page.
	 *
	 * @since 1.0.0
	 */
	public function add_action_links( $links ) {

		return array_merge(
			array(
				'settings' => '<a href="' . admin_url( 'options-general.php?page=' . $this->plugin_slug ) . '">' . __( 'Settings', $this->plugin_slug ) . '</a>'
			),
			$links
		);

	}

	/**
	 * Helper function for defining variables for the current page
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function get_settings() {

		$output = array();

		// put together the output array
		$output['option_name']       = $this->plugin_slug.'_options'; // the option name as used in the get_option() call.
		$output['page_sections']     = $this->options_page_sections(); // the setting section
		$output['page_fields']       = $this->options_page_fields(); // the setting fields

		return $output;
	}

	/**
	 * Register our setting
	 *
	 * @since 1.0.0
	 */
	public function tms_mailer_register_settings(){

		// get the settings sections array
		$settings_output    = $this->get_settings();
		$option_name = $settings_output['option_name'];
		$options = get_option($option_name);

		//setting
		//register_setting( $option_group, $option_name, $sanitize_callback );
		register_setting($option_name, $option_name, array( $this, 'validate_options' ) );

		//sections
		// add_settings_section( $id, $title, $callback, $page );
		if(!empty($settings_output['page_sections'])){
			// call the "add_settings_section" for each if 'rerouting' field is enabled!
				foreach ( $settings_output['page_sections'] as $id => $title ) {
					add_settings_section(
						$id,
						$title,
						array( $this, 'section_callback'),
						$this->plugin_slug
					);
				}
		}

		//fields
		if(!empty($settings_output['page_fields'])){
			// call the "add_settings_field" for each!
			foreach ($settings_output['page_fields'] as $option) {
				$this->create_settings_field($option);
			}
		}
	}

	/**
	 * Define our settings sections
	 *
	 * array key=$id, array value=$title in: add_settings_section( $id, $title, $callback, $page );
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function options_page_sections() {

		$sections = array();
		// $sections[$id]       = __($title, 'textdomain');
		$sections['activation_section']    = __( 'Activation Settings', $this->plugin_slug );
		$sections['main_section']    = __( 'Primary Settings', $this->plugin_slug );
		$sections['cron_section']    = __( 'Cron Settings', $this->plugin_slug );

		return $sections;
	}

	/**
	 * Section HTML, displayed before the first option
	 *
	 * @param string
	 * @since 1.0.0
	 */
	public function section_callback($desc) {
//		echo "<p>" . __( 'Settings for this section', $this->plugin_slug ) . "</p>";
	}

	/**
	 * Define our form fields (settings)
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function options_page_fields() {

		// Activation section
		$options[] = array(
			"section" => "activation_section",
			"id"      => $this->plugin_slug . "_rerouting_select",
			"title"   => __( 'Use TMS For Outbound Email', $this->plugin_slug ),
			"desc"    => __( 'If this option is enabled, emails from your Wordpress site will be sent using GovDelivery\'s Transactional Message Service (TMS).', $this->plugin_slug ),
			"type"    => "select",
			'std'     => '',
			"choices" => array(
				__( 'Disable', $this->plugin_slug ) . "|0",
				__( 'Enable', $this->plugin_slug ) . "|1"
			),
			"class"   => ''
		);

		// Main section
		$options[] = array(
			"section" => "main_section",
			"id"      => $this->plugin_slug . "_url",
			"title"   => __( 'GovDelivery TMS Account URL', $this->plugin_slug ),
			"desc"    => __( 'Enter the URL of your GovDelivery TMS instance. e.g. https://stage-tms.govdelivery.com or https://tms.govdelivery.com.', $this->plugin_slug ),
			"type"    => "text",
			'std'     => '',
			"class"   => "url"
		);

		$options[] = array(
			"section" => "main_section",
			"id"      => $this->plugin_slug . "_auth_token",
			"title"   => __( 'TMS Authentication Token', $this->plugin_slug ),
			"desc"    => 'Enter the authentication token needed to send emails through your GovDelivery TMS service. If you do not have an authentication token, please content your GovDelivery account representative.',
			"type"    => "text",
			'std'     => '',
			"class"   => ''
		);

		$options[] = array(
			"section" => "main_section",
			"id"      => $this->plugin_slug . "_from_name",
			"title"   => __( '"From" name', $this->plugin_slug ),
			"desc"    => __( 'The name displayed in the From field of the received email. E.G. John Smith.', $this->plugin_slug ),
			"type"    => "text",
			'std'     => '',
			"class"   => "nohtml"
		);

		$options[] = array(
			"section" => "main_section",
			"id"      => $this->plugin_slug . "_cron_select",
			"title"   => __( 'Cron Job', $this->plugin_slug ),
			"desc"    => __( 'Enable/Disable the Cron Job.', $this->plugin_slug ),
			"type"    => "select",
			'std'     => '',
			"choices" => array(
				__( 'Disable', $this->plugin_slug ) . "|0",
				__( 'Enable', $this->plugin_slug ) . "|1"
			),
			"class"   => ''
		);

		// Cron section
		$options[] = array(
			"section" => "cron_section",
			"id"      => $this->plugin_slug . "_queue_quantity",
			"title"   => __( 'Emails in queue', $this->plugin_slug ),
			"desc"    => __( 'Number of emails that will be sent in one queue with Cron.', $this->plugin_slug ),
			"type"    => "text",
			"std"    => "100",
			"class"   => "numeric"
		);

		$options[] = array(
			"section" => "cron_section",
			"id"      => $this->plugin_slug . "_cron_schedule",
			"title"   => __( 'Cron schedule', $this->plugin_slug ),
			"desc"    => __( 'Hitting "Save changes" will run the cron job (or will stop previous one and run a new one with new interval)".', $this->plugin_slug ),
			"type"    => "select",
			'std'     => '',
			"choices" => array(
				__( 'Every 5 minutes', $this->plugin_slug ) . "|five_minutes",
				__( 'Every 10 minutes', $this->plugin_slug ) . "|ten_minutes",
				__( 'Every 15 minutes', $this->plugin_slug ) . "|fifteen_minutes",
				__( 'Every 30 minutes', $this->plugin_slug ) . "|thirty_minutes",
				__( 'Every 45 minutes', $this->plugin_slug ) . "|fourty_five_minutes",
				__( 'Every hour', $this->plugin_slug ) . "|hourly",
				__( 'Every 2 hours', $this->plugin_slug ) . "|two_hours",
				__( 'Every 3 hours', $this->plugin_slug ) . "|three_hours",
				__( 'Every 6 hours', $this->plugin_slug ) . "|six_hours",
				__( 'Every 12 hours', $this->plugin_slug ) . "|twicedaily",
				__( 'Once a day', $this->plugin_slug ) . "|daily",
			),
			"class"   => ''
		);


    return $options;
	}

	/**
	 * Helper function for registering our form field settings
	 *
	 * src: http://alisothegeek.com/2011/01/wordpress-settings-api-tutorial-1/
	 * @params array $args The array of arguments to be used in creating the field
	 * @since 1.0.0
	 */
	public function create_settings_field( $args = array() ) {
		// default array to overwrite when calling the function
		$defaults = array(
			'id'      => 'default_field',                    // the ID of the setting in our options array, and the ID of the HTML form element
			'title'   => 'Default Field',                    // the label for the HTML form element
			'desc'    => 'This is a default description.',   // the description displayed under the HTML form element
			'std'     => '',                                 // the default value for this setting
			'type'    => 'text',                             // the HTML form element to use
			'section' => 'main_section',                     // the section this setting belongs to — must match the array key of a section in options_page_sections()
			'choices' => array(),                            // (optional): the values in radio buttons or a drop-down menu
			'class'   => ''                                  // the HTML form element class. Also used for validation purposes!
		);

		/**
		 * "extract" to be able to use the array keys as variables in our function output below
		 *
		 * @var $id string
		 * @var $title string
		 * @var $desc string
		 * @var $std string
		 * @var $type string
		 * @var $section string
		 * @var $choices array
		 * @var $class string
		 */
		extract( wp_parse_args( $args, $defaults ) );

		// additional arguments for use in form field output in the function form_field_fn!
		$field_args = array(
			'type'      => $type,
			'id'        => $id,
			'desc'      => $desc,
			'std'       => $std,
			'choices'   => $choices,
			'label_for' => $id,
			'class'     => $class
		);

		add_settings_field(
			$id,
			$title,
			array( $this, 'form_field_fn' ),
			$this->plugin_slug,
			$section,
			$field_args
		);
	}

	/**
	* Form Fields HTML
	* All form field types share the same function!!
	*
	* @param array
	* @since 1.0.0
	*/
	public function form_field_fn($args = array()) {

		/**
		 * "extract" to be able to use the array keys as variables in our function output below
		 *
		 * @var $id string
		 * @var $title string
		 * @var $desc string
		 * @var $std string
		 * @var $type string
		 * @var $section string
		 * @var $choices array
		 * @var $class string
		 */
		extract( $args );

		// get the settings sections array
		$settings_output    = $this->get_settings();

		$option_name = $settings_output['option_name'];
		$options            = get_option($option_name);

		// pass the standard value if the option is not yet set in the database
		if ( !isset( $options[$id] ) && 'type' != 'checkbox' ) {
			$options[$id] = $std;
		}

		// additional field class. output only if the class is defined in the create_setting arguments
		$field_class = ($class != '') ? ' ' . $class : '';


		// switch html display based on the setting type.
		switch ( $type ) {
			case 'text':
				$options[$id] = stripslashes($options[$id]);
				$options[$id] = esc_attr( $options[$id]);
				echo "<input class='regular-text$field_class' type='text' id='$id' name='" . $option_name . "[$id]' value='$options[$id]' />";
				echo ($desc != '') ? '<br /><span class=\'description\'>'.$desc.'</span>' : "";
				break;

			case 'select':
				echo "<select id='$id' class='select$field_class' name='" . $option_name . "[$id]'>";
				foreach($choices as $item) {

					$item = explode("|",$item);
					$item[0] = esc_html($item[0], $this->plugin_slug);

					$selected = ($options[$id]==$item[1]) ? 'selected="selected"' : '';
					echo "<option value='$item[1]' ".$selected.">$item[0]</option>";
				}
				echo "</select>";
				echo ($desc != '') ? '<br /><span class=\'description\'>'.$desc.'</span>' : "";
				break;

			case "multi-text":
				foreach($choices as $item) {
					$item = explode("|",$item); // cat_name|cat_slug
					$item[0] = esc_html__($item[0], $this->plugin_slug);
					$value = '';
					if (!empty($options[$id])) {
						foreach ($options[$id] as $option_key => $option_val){
							if ($item[1] == $option_key) {
								$value = $option_val;
							}
						}
					}
					echo "<span>$item[0]:</span> <input class='$field_class' type='text' id='$id|$item[1]' name='" . $option_name . "[$id|$item[1]]' value='$value' /><br/>";
				}
				echo ($desc != '') ? '<span class=\'description\'>'.$desc.'</span>' : "";
				break;

			case 'textarea':
				$options[$id] = stripslashes($options[$id]);
				$options[$id] = esc_html( $options[$id]);
				echo "<textarea class='textarea$field_class' type='text' id='$id' name='" . $option_name . "[$id]' rows='5' cols='30'>$options[$id]</textarea>";
				echo ($desc != '') ? "<br /><span class='description'>'.$desc.'</span>" : "";
				break;

			case 'checkbox':
				echo "<input class='checkbox$field_class' type='checkbox' id='$id' name='" . $option_name . "[$id]' value='1' " . checked( $options[$id], 1, false ) . " />";
				echo ($desc != '') ? "<br /><span class='description'>'.$desc.'</span>" : "";
				break;

			case "multi-checkbox":
				foreach($choices as $item) {

					$item = explode("|",$item);
					$item[0] = esc_html($item[0], $this->plugin_slug);

					$checked = '';

					if ( isset($options[$id][$item[1]]) ) {
						if ( $options[$id][$item[1]] == 'true') {
							$checked = 'checked="checked"';
						}
					}

					echo "<input class='checkbox$field_class' type='checkbox' id='$id|$item[1]' name='" . $option_name . "[$id|$item[1]]' value='1' $checked /> $item[0] <br/>";
				}
				echo ($desc != '') ? "<br /><span class='description'>'.$desc.'</span>" : "";
				break;
		}
	}

	/**
	 * Validate each setting field as needed
	 *
	 * @param array $input Contains all settings fields as array keys
	 * @since 1.0.0
	 * @return array
	 */
	function validate_options($input) {

		// for enhanced security, create a new empty array
		$valid_input = array();

		// collect only the values we expect and fill the new $valid_input array i.e. whitelist our option IDs

		// get the settings sections array
		$settings_output = $this->get_settings();

		$options = $settings_output['page_fields'];

		// run a foreach and switch on option type
		foreach ($options as $option) {

			switch ( $option['type'] ) {
				case 'text':
					//switch validation based on the class!
					switch ( $option['class'] ) {
						//for numeric
						case 'numeric':
							//accept the input only when numeric!
							$input[$option['id']]       = trim($input[$option['id']]); // trim whitespace
							$valid_input[$option['id']] = (is_numeric($input[$option['id']])) ? $input[$option['id']] : '';

							// register error
							if(is_numeric($input[$option['id']]) == FALSE && $input[$option['id']] != "") {
								add_settings_error(
									$option['id'], // setting title
										$this->plugin_slug . '_txt_numeric_error', // error ID
									__('Expecting a Numeric value! Please fix.',$this->plugin_slug), // error message
									'error '.$option['id'] // type of message
								);
							}
							break;

						//for multi-numeric values (separated by a comma)
						case 'multinumeric':
							//accept the input only when the numeric values are comma separated
							$input[$option['id']]       = trim($input[$option['id']]); // trim whitespace

							if($input[$option['id']] !=''){
								// /^-?\d+(?:,\s?-?\d+)*$/ matches: -1 | 1 | -12,-23 | 12,23 | -123, -234 | 123, 234  | etc.
								$valid_input[$option['id']] = (preg_match('/^-?\d+(?:,\s?-?\d+)*$/', $input[$option['id']]) == 1) ? $input[$option['id']] : '';
							}else{
								$valid_input[$option['id']] = $input[$option['id']];
							}

							// register error
							if($input[$option['id']] !='' && preg_match('/^-?\d+(?:,\s?-?\d+)*$/', $input[$option['id']]) != 1) {
								add_settings_error(
									$option['id'], // setting title
										$this->plugin_slug . '_txt_multinumeric_error', // error ID
									__('Expecting comma separated numeric values! Please fix.',$this->plugin_slug), // error message
									'error '.$option['id'] // type of message
								);
							}
							break;

						//for no html
						case 'nohtml':
							//accept the input only after stripping out all html, extra white space etc!
							$input[$option['id']]       = sanitize_text_field($input[$option['id']]); // need to add slashes still before sending to the database
							$valid_input[$option['id']] = addslashes($input[$option['id']]);
							break;

						//for url
						case 'url':
							//accept the input only when the url has been sanited for database usage with esc_url_raw()
							$input[$option['id']]       = trim($input[$option['id']]); // trim whitespace
							if($input[$option['id']] != ''){
								$valid_protocols = array('https');
								$valid_input[$option['id']] = esc_url_raw(untrailingslashit($input[$option['id']]), $valid_protocols);
								if($valid_input[$option['id']] == '') {
									add_settings_error(
										$option['id'], // setting title
										$this->plugin_slug . '_txt_url_error', // error ID
										__('Please enter a valid url address. Acceptable protocols: ' . implode(', ', $valid_protocols),$this->plugin_slug), // error message
										'error '.$option['id'] // type of message
									);
								}
							}
							break;

						//for email
						case 'email':
							//accept the input only after the email has been validated
							$input[$option['id']]       = trim($input[$option['id']]); // trim whitespace
							if($input[$option['id']] != ''){
								$valid_input[$option['id']] = (is_email($input[$option['id']])!== FALSE) ? $input[$option['id']] : __('Invalid email! Please re-enter!',$this->plugin_slug);
							}elseif($input[$option['id']] == ''){
								$valid_input[$option['id']] = '';
							}

							// register error
							if(is_email($input[$option['id']])== FALSE || $input[$option['id']] == '') {
								add_settings_error(
									$option['id'], // setting title
										$this->plugin_slug . '_txt_email_error', // error ID
									__('Please enter a valid email address.',$this->plugin_slug), // error message
									'error '.$option['id'] // type of message
								);
							}
							break;

						// a "cover-all" fall-back when the class argument is not set
						default:
							// accept only a few inline html elements
							$allowed_html = array(
								'a' => array('href' => array (),'title' => array ()),
								'b' => array(),
								'em' => array (),
								'i' => array (),
								'strong' => array()
							);

							$input[$option['id']]       = trim($input[$option['id']]); // trim whitespace
							$input[$option['id']]       = force_balance_tags($input[$option['id']]); // find incorrectly nested or missing closing tags and fix markup
							$input[$option['id']]       = wp_kses( $input[$option['id']], $allowed_html); // need to add slashes still before sending to the database
							$valid_input[$option['id']] = addslashes($input[$option['id']]);
							break;
					}
					break;

				case "multi-text":
					// this will hold the text values as an array of 'key' => 'value'
					unset($textarray);

					$text_values = array();
					foreach ($option['choices'] as $v ) {
						// explode the connective
						$pieces = explode("|", $v);

						$text_values[] = $pieces[1];
					}

					foreach ($text_values as $v ) {

						// Check that the option isn't empty
						if (!empty($input[$option['id'] . '|' . $v])) {
							// If it's not null, make sure it's sanitized, add it to an array
							switch ($option['class']) {
								// different sanitation actions based on the class create you own cases as you need them

								//for numeric input
								case 'numeric':
									//accept the input only if is numberic!
									$input[$option['id'] . '|' . $v]= trim($input[$option['id'] . '|' . $v]); // trim whitespace
									$input[$option['id'] . '|' . $v]= (is_numeric($input[$option['id'] . '|' . $v])) ? $input[$option['id'] . '|' . $v] : '';
									break;

								// a "cover-all" fall-back when the class argument is not set
								default:
									// strip all html tags and white-space.
									$input[$option['id'] . '|' . $v]= sanitize_text_field($input[$option['id'] . '|' . $v]); // need to add slashes still before sending to the database
									$input[$option['id'] . '|' . $v]= addslashes($input[$option['id'] . '|' . $v]);
									break;
							}
							// pass the sanitized user input to our $textarray array
							$textarray[$v] = $input[$option['id'] . '|' . $v];

						} else {
							$textarray[$v] = '';
						}
					}
					// pass the non-empty $textarray to our $valid_input array
					if (!empty($textarray)) {
						$valid_input[$option['id']] = $textarray;
					}
					break;

				case 'textarea':
					//switch validation based on the class!
					switch ( $option['class'] ) {
						//for only inline html
						case 'inlinehtml':
							// accept only inline html
							$input[$option['id']]       = trim($input[$option['id']]); // trim whitespace
							$input[$option['id']]       = force_balance_tags($input[$option['id']]); // find incorrectly nested or missing closing tags and fix markup
							$input[$option['id']]       = addslashes($input[$option['id']]); //wp_filter_kses expects content to be escaped!
							$valid_input[$option['id']] = wp_filter_kses($input[$option['id']]); //calls stripslashes then addslashes
							break;

						//for no html
						case 'nohtml':
							//accept the input only after stripping out all html, extra white space etc!
							$input[$option['id']]       = sanitize_text_field($input[$option['id']]); // need to add slashes still before sending to the database
							$valid_input[$option['id']] = addslashes($input[$option['id']]);
							break;

						//for allowlinebreaks
						case 'allowlinebreaks':
							//accept the input only after stripping out all html, extra white space etc!
							$input[$option['id']]       = wp_strip_all_tags($input[$option['id']]); // need to add slashes still before sending to the database
							$valid_input[$option['id']] = addslashes($input[$option['id']]);
							break;

						// a "cover-all" fall-back when the class argument is not set
						default:
							// accept only limited html
							//my allowed html
							$allowed_html = array(
								'a'             => array('href' => array (),'title' => array ()),
								'b'             => array(),
								'blockquote'    => array('cite' => array ()),
								'br'            => array(),
								'dd'            => array(),
								'dl'            => array(),
								'dt'            => array(),
								'em'            => array (),
								'i'             => array (),
								'li'            => array(),
								'ol'            => array(),
								'p'             => array(),
								'q'             => array('cite' => array ()),
								'strong'        => array(),
								'ul'            => array(),
								'h1'            => array('align' => array (),'class' => array (),'id' => array (), 'style' => array ()),
								'h2'            => array('align' => array (),'class' => array (),'id' => array (), 'style' => array ()),
								'h3'            => array('align' => array (),'class' => array (),'id' => array (), 'style' => array ()),
								'h4'            => array('align' => array (),'class' => array (),'id' => array (), 'style' => array ()),
								'h5'            => array('align' => array (),'class' => array (),'id' => array (), 'style' => array ()),
								'h6'            => array('align' => array (),'class' => array (),'id' => array (), 'style' => array ())
							);

							$input[$option['id']]       = trim($input[$option['id']]); // trim whitespace
							$input[$option['id']]       = force_balance_tags($input[$option['id']]); // find incorrectly nested or missing closing tags and fix markup
							$input[$option['id']]       = wp_kses( $input[$option['id']], $allowed_html); // need to add slashes still before sending to the database
							$valid_input[$option['id']] = addslashes($input[$option['id']]);
							break;
					}
					break;

				case 'select':
					// process $select_values
					$select_values = array();
					foreach ($option['choices'] as $v) {
						// explode the connective
						$pieces = explode("|", $v);

						$select_values[] = $pieces[1];
					}
					// check to see if selected value is in our approved array of values!
					$valid_input[$option['id']] = (in_array( $input[$option['id']], $select_values) ? $input[$option['id']] : '' );
					break;

				case 'checkbox':
					// if it's not set, default to null!
					if (!isset($input[$option['id']])) {
						$input[$option['id']] = null;
					}
					// Our checkbox value is either 0 or 1
					$valid_input[$option['id']] = ( $input[$option['id']] == 1 ? 1 : 0 );
					break;

				case 'multi-checkbox':
					unset($checkboxarray);
					$check_values = array();
					foreach ($option['choices'] as $v ) {
						// explode the connective
						$pieces = explode("|", $v);

						$check_values[] = $pieces[1];
					}

					foreach ($check_values as $v ) {

						// Check that the option isn't null
						if (!empty($input[$option['id'] . '|' . $v])) {
							// If it's not null, make sure it's true, add it to an array
							$checkboxarray[$v] = 'true';
						}
						else {
							$checkboxarray[$v] = 'false';
						}
					}
					// Take all the items that were checked, and set them as the main option
					if (!empty($checkboxarray)) {
						$valid_input[$option['id']] = $checkboxarray;
					}
					break;

			}
		}
		return $valid_input; // return validated input
	}

	/**
	 * Run/stop cron schedule if settings have been saved.
	 *
	 * @since 1.0.0
	 */
	public function run_cron_job() {
		// if plugin settings have been saved
		if ( isset($_GET['settings-updated']) && $_GET['settings-updated'] && $this->is_tms_enabled === true ) {

			$timestamp = wp_next_scheduled( 'tms_cron_job' );
			if ($this->is_cron_enabled === true || $this->is_cron_enabled === 'need_reactivate') {

				$options = get_option($this->plugin_slug.'_options');

				//Use wp_next_scheduled to check if the event is already scheduled
				$timestamp = wp_next_scheduled( 'tms_cron_job' );

				//If $timestamp == false schedule the event since it hasn't been done previously
				if( $timestamp !== false ){
					wp_clear_scheduled_hook( 'tms_cron_job' );
				}

				//Schedule the event for right now, then to repeat using the hook 'tms_cron_job'
				wp_schedule_event( time(), $options[$this->plugin_slug.'_cron_schedule'], 'tms_cron_job' );

			} else {
				if ( $timestamp !== false ) {
					wp_clear_scheduled_hook( 'tms_cron_job' );
					$plugin = TMS_Mailer::get_instance();
					$plugin->send_emails_before_cron_disable();
				}
			}
		}
	}

	/**
	 * Add a top-level menu item to the bottom of the admin menus if cron is enabled
	 *
	 * @since 1.0.0
	 */
	public function add_reports_admin_menu() {
		if ( $this->is_cron_enabled === true ) {
			add_menu_page(
				__( 'GD Subscriptions', $this->plugin_slug ),
				__( 'GD Subscriptions', $this->plugin_slug ),
				'manage_options',
					$this->plugin_slug.'_reports',
				array( $this, 'render_reports_list_page' )
			);
		}
	}

	/**
	 * This function renders the admin page and list table. Although it's
	 * possible to call prepare_items() and display() from the constructor, there
	 * are often times where you may need to include logic here between those steps,
	 * so we've instead called those methods explicitly. It keeps things flexible, and
	 * it's the way the list tables are used in the WordPress core.
	 *
	 * @since 1.0.0
	 */
	public function render_reports_list_page(){

		//Create an instance of our package class...
		$class_list_table = new TMS_Mailer_List_Table();
		//Fetch, prepare, sort, and filter our data...
		$class_list_table->prepare_items();

		?>
		<div class="wrap">

			<div id="icon-users" class="icon32"><br/></div>
			<h2><?php echo esc_html( get_admin_page_title() ); ?></h2>

			<!-- Forms are NOT created automatically, so you need to wrap the table in one to use features like bulk actions -->
			<form id="emails-filter" method="get">
				<!-- For plugins, we also need to ensure that the form posts back to our current page -->
				<input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>" />
				<!-- Now we can render the completed list table -->
				<?php $class_list_table->display() ?>
			</form>

		</div>
	<?php
	}

	/**
	 * Send test email on page load if user submits form
	 *
	 * @since 1.0.0
	 */
	public function send_test_email() {

		if ( isset($_POST['action']) && $_POST['action'] == 'send-test-email' && $this->is_tms_enabled === true ) {

			//user posted variables
			$to = isset($_POST['tms-mailer_test_email']) ? $_POST['tms-mailer_test_email'] : '';
			$subject = isset($_POST['tms-mailer_test_subject']) ? $_POST['tms-mailer_test_subject'] : '';
			$message = isset($_POST['tms-mailer_test_body']) ? $_POST['tms-mailer_test_body'] : '';


			//validate email
			if(!filter_var($to, FILTER_VALIDATE_EMAIL))
				add_settings_error(
					'tms-mailer_test_email_error', // setting title
						$this->plugin_slug . '_test_email_error', // error ID
					__('Email Address Invalid.', $this->plugin_slug), // error message
					'error tms-mailer_test_email' // type of message
				);
			else //email is valid
			{
				//validate presence of name and message
				if(empty($subject) || empty($message)){
					add_settings_error(
						'tms-mailer_missing_test_content', // setting title
							$this->plugin_slug . '_test_email_error', // error ID
						__('Please supply all information in order to send test email.', $this->plugin_slug), // error message
						'error' // type of message
					);
				}
				else //ready to go!
				{
					$sent = wp_mail($to, $subject, strip_tags($message));
					if($sent) {
						//message sent!
						add_settings_error(
							'tms-mailer_test_email_success', // setting title
								$this->plugin_slug . '_test_email_success', // error ID
							__('Thanks! Your message has been sent.', $this->plugin_slug), // error message
							'updated' // type of message
						);
					}
					else {
						//message wasn't sent
						add_settings_error(
							'tms-mailer_test_email_error', // setting title
								$this->plugin_slug . '_test_email_error', // error ID
							__('Message was not sent. Try Again.', $this->plugin_slug), // error message
							'error' // type of message
						);
					}
				}
			}
		}
	}

}