<?php
/**
 * TMS Mailer.
 *
 * @package   TMS_Mailer
 * @author    Spyder Trap
 * @license   GPL-2.0+
 * @link      http://spydertrap.com
 * @copyright 2013 Spyder Trap
 */

 /**
 * The WP_List_Table class isn't automatically available to plugins, so we need
 * to check if it's available and load it if necessary.
 */
if(!class_exists('WP_List_Table')){
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}



/**
 * Create a new list table package that extends the core WP_List_Table class.
 * This class is used to represent a list table of emails in queues.
 *
 * @package   TMS_Mailer
 * @author    Spyder Trap <admin@spydertrap.com>
 */
class TMS_Mailer_List_Table extends WP_List_Table {

	/**
	* Initialize the plugin by setting a constructor that references the parent constructor.
	*
	* @since 1.0.0
	*/
	function __construct(){
		global $status, $page;

		//Set parent defaults
		parent::__construct( array(
			'singular'  => 'email',     //singular name of the listed records
			'plural'    => 'emails',    //plural name of the listed records
			'ajax'      => false        //does this table support ajax?
		) );

	}


	/**
	 * Recommended. This method is called when the parent class can't find a method
	 * specifically build for a given column. Generally, it's recommended to include
	 * one method for each column you want to render, keeping your package class
	 * neat and organized. For example, if the class needs to process a column
	 * named 'id', it would first see if a method named $this->column_id()
	 * exists - if it does, that method will be used. If it doesn't, this one will
	 * be used. Generally, you should try to use custom column methods as much as
	 * possible.
	 *
	 * Since we have defined a column_id() method later on, this method doesn't
	 * need to concern itself with any column with a name of 'id'. Instead, it
	 * needs to handle everything else.
	 *
	 * For more detailed insight into how columns are handled, take a look at
	 * WP_List_Table::single_row_columns()
	 *
	 * @param array $item A singular item (one full row's worth of data)
	 * @param string $column_name The name/slug of the column to be processed
	 * @since 1.0.0
	 * @return string Text or HTML to be placed inside the column <td>
	 */
	function column_default($item, $column_name){
		switch($column_name){
			case 'email':
			case 'subject':
			case 'body':
				return $item[$column_name];
			default:
				return print_r($item,true); //Show the whole array for troubleshooting purposes
		}
	}


	/**
	 * Recommended. This is a custom column method and is responsible for what
	 * is rendered in any column with a name/slug of 'id'. Every time the class
	 * needs to render a column, it first looks for a method named
	 * column_{$column_id} - if it exists, that method is run. If it doesn't
	 * exist, column_default() is called instead.
	 *
	 * @see WP_List_Table::::single_row_columns()
	 * @param array $item A singular item (one full row's worth of data)
	 * @since 1.0.0
	 * @return string Text to be placed inside the column <td> (movie title only)
	 */
	function column_id($item){

		//Build row actions
		$actions = array(
			'force_send'      => sprintf('<a href="?page=%s&action=%s&email=%s">Force send</a>',$_REQUEST['page'],'force_send',$item['id']),
			'delete'    => sprintf('<a href="?page=%s&action=%s&email=%s">Delete</a>',$_REQUEST['page'],'delete',$item['id']),
		);

		//Return the title contents
		return sprintf('%1$s %2$s',
			/*$1%s*/ $item['id'],
			/*$2%s*/ $this->row_actions($actions)
		);
	}

	/**
	 * REQUIRED if displaying checkboxes or using bulk actions! The 'cb' column
	 * is given special treatment when columns are processed. It ALWAYS needs to
	 * have it's own method.
	 *
	 * @see WP_List_Table::::single_row_columns()
	 * @param array $item A singular item (one full row's worth of data)
	 * @since 1.0.0
	 * @return string Text to be placed inside the column <td> (movie title only)
	 */
	function column_cb($item){
		return sprintf(
			'<input type="checkbox" name="%1$s[]" value="%2$s" />',
			/*$1%s*/ $this->_args['singular'],  //Let's simply repurpose the table's singular label
			/*$2%s*/ $item['id']                //The value of the checkbox should be the record's id
		);
	}


	/**
	 * REQUIRED! This method dictates the table's columns and titles. This should
	 * return an array where the key is the column slug (and class) and the value
	 * is the column's title text. If you need a checkbox for bulk actions, refer
	 * to the $columns array below.
	 *
	 * The 'cb' column is treated differently than the rest. If including a checkbox
	 * column in your table you must create a column_cb() method. If you don't need
	 * bulk actions or checkboxes, simply leave the 'cb' entry out of your array.
	 *
	 * @see WP_List_Table::::single_row_columns()
	 * @since 1.0.0
	 * @return array An associative array containing column information: 'slugs'=>'Visible Titles'
	 */
	function get_columns(){
		$columns = array(
			'cb'        => '<input type="checkbox" />', //Render a checkbox instead of text
			'id'     => 'Unique id',
			'email'     => 'Recipient email',
			'subject'    => 'Subject',
			'body'  => 'Body'
		);
		return $columns;
	}

	/**
	 * Optional. If you want one or more columns to be sortable (ASC/DESC toggle),
	 * you will need to register it here. This should return an array where the
	 * key is the column that needs to be sortable, and the value is db column to
	 * sort by. Often, the key and value will be the same, but this is not always
	 * the case (as the value is a column name from the database, not the list table).
	 *
	 * This method merely defines which columns should be sortable and makes them
	 * clickable - it does not handle the actual sorting. You still need to detect
	 * the ORDERBY and ORDER querystring variables within prepare_items() and sort
	 * your data accordingly (usually by modifying your query).
	 *
	 * @since 1.0.0
	 * @return array An associative array containing all the columns that should be sortable: 'slugs'=>array('data_values',bool)
	 */
	function get_sortable_columns() {
		$sortable_columns = array(
			'id'     => array('id',false),     //true means it's already sorted
			'email'     => array('email',false),     //true means it's already sorted
			'subject'    => array('subject',false),
			'body'  => array('body',false)
		);
		return $sortable_columns;
	}


	/**
	 * Optional. If you need to include bulk actions in your list table, this is
	 * the place to define them. Bulk actions are an associative array in the format
	 * 'slug'=>'Visible Title'
	 *
	 * If this method returns an empty value, no bulk action will be rendered. If
	 * you specify any bulk actions, the bulk actions box will be rendered with
	 * the table automatically on display().
	 *
	 * @since 1.0.0
	 * @return array An associative array containing all the bulk actions: 'slugs'=>'Visible Titles'
	 **/
	function get_bulk_actions() {
		$actions = array(
			'force_send'    => 'Force send',
			'delete'    => 'Delete'
		);
		return $actions;
	}

	/**
	 * Get current element selected for the bulk actions events.
	 *
	 * @since 1.0.0
	 *
	 * @return string|bool The action name or False if no action was selected
	 */
	function current_action_email_id() {
		if ( isset( $_REQUEST['email'] ) && -1 != $_REQUEST['email'] )
			return $_REQUEST['email'];

		return false;
	}

	/**
	 * Optional. Handle bulk actions.
	 *
	 * @see $this->prepare_items()
	 * @since 1.0.0
	 */
	function process_bulk_action() {
		global $wpdb;

		$table_name = $wpdb->prefix . "tms_mails";

		// Detect element ids for actions to do
		$ids = $this->current_action_email_id();

		if ($ids == false)
			return;

		//Detect when a bulk action is being triggered...
		if( 'delete'===$this->current_action() ) {

			// delete email/emails from db
			$sql_delete = $wpdb->prepare("DELETE FROM $table_name WHERE id IN (". str_repeat("%d,", count($ids)-1) . "%d)" , $ids);
			$wpdb->query($sql_delete);
		}

		if( 'force_send'===$this->current_action() ) {
			// get records from db by id and force send them
			$sql_select = $wpdb->prepare("SELECT * FROM $table_name WHERE id IN (". str_repeat("%d,", count($ids)-1) . "%d)" , $ids);
			$rows = $wpdb->get_results( $sql_select );

			$rows_count = count($rows);

			for($i = 0; $i < $rows_count; $i++) {
				wp_mail($rows[$i]->email, $rows[$i]->subject, $rows[$i]->body, false, false, true);
			}

			// delete email/emails from db
			$sql_delete = $wpdb->prepare("DELETE FROM $table_name WHERE id IN (". str_repeat("%d,", count($ids)-1) . "%d)" , $ids);
			$wpdb->query($sql_delete);
		}

	}


	/**
	 * REQUIRED! This is where you prepare your data for display. This method will
	 * usually be used to query the database, sort and filter the data, and generally
	 * get it ready to be displayed. At a minimum, we should set $this->items and
	 * $this->set_pagination_args(), although the following properties and methods
	 * are frequently interacted with here...
	 *
	 * @global WPDB $wpdb
	 * @uses $this->_column_headers
	 * @uses $this->items
	 * @uses $this->get_columns()
	 * @uses $this->get_sortable_columns()
	 * @uses $this->get_pagenum()
	 * @uses $this->set_pagination_args()
	 * @since 1.0.0
	 */
	function prepare_items() {
		global $wpdb;

		/**
		 * Get plugins options
		 */
		$options = get_option('tms-mailer_options');

		/**
		 * Set table name and create a db query
		 */
		$table_name = $wpdb->prefix . "tms_mails";
		$query = "SELECT * FROM $table_name";

		/**
		 * Show per page the whole queue
		 */
		$per_page = $options['tms-mailer_queue_quantity'];


		/**
		 * REQUIRED. Now we need to define our column headers. This includes a complete
		 * array of columns to be displayed (slugs & titles), a list of columns
		 * to keep hidden, and a list of columns that are sortable.
		 */
		$columns = $this->get_columns();
		$hidden = array();
		$sortable = $this->get_sortable_columns();


		/**
		 * REQUIRED. Finally, we build an array to be used by the class for column
		 * headers. The $this->_column_headers property takes an array which contains
		 * 3 other arrays. One for all columns, one for hidden columns, and one
		 * for sortable columns.
		 */
		$this->_column_headers = array($columns, $hidden, $sortable);


		/**
		 * Optional. You can handle your bulk actions however you see fit. In this
		 * case, we'll handle them within our package just to keep things clean.
		 */
		$this->process_bulk_action();

		/**
		 * This checks for sorting input and sorts the data in our array accordingly.
		 */
		$orderby = !empty($_REQUEST["orderby"]) ? mysql_real_escape_string($_REQUEST["orderby"]) : 'ASC';
		$order = !empty($_REQUEST["order"]) ? mysql_real_escape_string($_REQUEST["order"]) : '';
		if(!empty($orderby) & !empty($order)){ $query.=' ORDER BY '.$orderby.' '.$order; }

		/**
		 * Get sorted rows from db
		 */
		$data = $wpdb->get_results( $query, ARRAY_A );

		/**
		 * REQUIRED for pagination. Let's figure out what page the user is currently
		 * looking at. We'll need this later, so you should always include it in
		 * your own package classes.
		 */
		$current_page = $this->get_pagenum();

		/**
		 * REQUIRED for pagination. Let's check how many items are in our data array.
		 * In real-world use, this would be the total number of items in your database,
		 * without filtering. We'll need this later, so you should always include it
		 * in your own package classes.
		 */
		$total_items = count($data);


		/**
		 * The WP_List_Table class does not handle pagination for us, so we need
		 * to ensure that the data is trimmed to only the current page. We can use
		 * array_slice()
		 */
		$data = array_slice($data,(($current_page-1)*$per_page),$per_page);



		/**
		 * REQUIRED. Now we can add our *sorted* data to the items property, where
		 * it can be used by the rest of the class.
		 */
		$this->items = $data;


		/**
		 * REQUIRED. We also have to register our pagination options & calculations.
		 */
		$this->set_pagination_args( array(
			'total_items' => $total_items,                  //WE have to calculate the total number of items
			'per_page'    => $per_page,                     //WE have to determine how many items to show on a page
			'total_pages' => ceil($total_items/$per_page)   //WE have to calculate the total number of pages
		) );
	}

}