<?php
/**
 * TMS Mailer.
 *
 * @package   TMS_Mailer
 * @author    Spyder Trap
 * @license   GPL-2.0+
 * @link      http://spydertrap.com
 * @copyright 2013 Spyder Trap
 */

if ( $plugin_tms_class->is_tms_enabled() === true && ! function_exists('wp_mail') ) :
	/**
	 * Send mail through TMS GovDelivery API.
	 *
	 * A true return value does not automatically mean that the user received the
	 * email successfully. It just only means that the method used was able to
	 * process the request without any errors.
	 *
	 * @TODO: Mail should include (doesn't supported by TMS on 12.6.2013):
	 * @TODO: Content-Type and charset.
	 * @TODO: Attachments.
	 * @TODO: Headers.
	 * @TODO: From email.
	 *
	 * @since 1.0.0
	 * @uses apply_filters() Calls 'wp_mail' hook on an array of all of the parameters.
	 * @uses apply_filters() Calls 'wp_mail_from_name' hook to get the from address name.
	 *
	 * @param string|array $to Array or comma-separated list of email addresses to send message.
	 * @param string $subject Email subject
	 * @param string $message Message contents
	 * @param string|array $headers Optional. Additional headers.
	 * @param string|array $attachments Optional. Files to attach.
	 * @param bool $is_cron Optional. Whether function run by CRON.
	 * @return bool Whether the email contents were sent successfully.
	 */
	function wp_mail( $to, $subject, $message, $headers = '', $attachments = array(), $is_cron = false ) {

		global $plugin_tms_class;

		// Get plugin options and plugin slug
		$plugin_slug = $plugin_tms_class->get_plugin_slug();
		$options = get_option($plugin_slug.'_options');

		//
		$base_url = $options[$plugin_slug.'_url'];
		$auth_token = $options[$plugin_slug.'_auth_token'];
		$from_name = $options[$plugin_slug.'_from_name'];
		$email_path = '/messages/email';
		$tms_url = $base_url . $email_path;

		$body = array();

		// Compact the input, apply the filters, and extract them back out
		extract( apply_filters( 'wp_mail', compact( 'to', 'subject', 'message', 'headers', 'attachments' ) ) );

		// Get cc and bcc emails from headers
		if ( !empty( $headers ) ) {
			if ( !is_array( $headers ) ) {
				// Explode the headers out, so this function can take both
				// string headers and an array of headers.
				$tempheaders = explode( "\n", str_replace( "\r\n", "\n", $headers ) );
			} else {
				$tempheaders = $headers;
			}
			$cc = array();
			$bcc = array();

			// If it's actually got contents
			if ( !empty( $tempheaders ) ) {
				// Iterate through the raw headers
				foreach ( (array) $tempheaders as $header ) {
					if ( strpos($header, ':') === false ) {
						if ( false !== stripos( $header, 'boundary=' ) ) {
							$parts = preg_split('/boundary=/i', trim( $header ) );
							$boundary = trim( str_replace( array( "'", '"' ), '', $parts[1] ) );
						}
						continue;
					}
					// Explode them out
					list( $name, $content ) = explode( ':', trim( $header ), 2 );

					// Cleanup crew
					$name    = trim( $name    );
					$content = trim( $content );

					switch ( strtolower( $name ) ) {
						case 'cc':
							$cc = array_merge( (array) $cc, explode( ',', $content ) );
							break;
						case 'bcc':
							$bcc = array_merge( (array) $bcc, explode( ',', $content ) );
							break;
					}
				}
			}
		}

		// Set destination addresses
		if ( !is_array( $to ) )
			$to = explode( ',', $to );

		// Collect recipients into array
		$recipients = array();
		foreach ( (array) $to as $recipient ) {
			try {
				// Break $recipient into name and address parts if in the format "Foo <bar@baz.com>"
				// @TODO: recipient name (doesn't supported by TMS on 12.6.2013).
				// $recipient_name = '';
				if( preg_match( '/(.*)<(.+)>/', $recipient, $matches ) ) {
					if ( count( $matches ) == 3 ) {
						// $recipient_name = $matches[1];
						$recipient = $matches[2];
					}
				}
				$recipients[] = $recipient;
			} catch ( ErrorException $e ) {
				continue;
			}
		}

		// Add any CC and BCC recipients
		if ( !empty( $cc ) ) {
			foreach ( (array) $cc as $recipient ) {
				try {
					// Break $recipient into name and address parts if in the format "Foo <bar@baz.com>"
					// @TODO: recipient name (doesn't supported by TMS on 12.6.2013).
					// $recipient_name = '';
					if( preg_match( '/(.*)<(.+)>/', $recipient, $matches ) ) {
						if ( count( $matches ) == 3 ) {
							// $recipient_name = $matches[1];
							$recipient = $matches[2];
						}
					}
					$recipients[] = $recipient;
				} catch ( ErrorException $e ) {
					continue;
				}
			}
		}

		if ( !empty( $bcc ) ) {
			foreach ( (array) $bcc as $recipient) {
				try {
					// Break $recipient into name and address parts if in the format "Foo <bar@baz.com>"
					// @TODO: recipient name (doesn't supported by TMS on 12.6.2013).
					// $recipient_name = '';
					if( preg_match( '/(.*)<(.+)>/', $recipient, $matches ) ) {
						if ( count( $matches ) == 3 ) {
							// $recipient_name = $matches[1];
							$recipient = $matches[2];
						}
					}
					$recipients[] = $recipient;
				} catch ( ErrorException $e ) {
					continue;
				}
			}
		}

		// If cron is enabled and function executed not by cron
		if ( $plugin_tms_class->is_cron_enabled() === true && !$is_cron ) {
			global $wpdb;

			$table_name = $wpdb->prefix . "tms_mails";

			foreach($recipients as $recipient) {
				$rows_affected = $wpdb->insert( $table_name, array( 'email' => $recipient, 'subject' => $subject, 'body' => $message, 'time' => current_time('mysql') ) );
			}
			return $rows_affected;
		}

		// From name
		// If we don't have a name from the plugin settings
		if ( !isset( $from_name ) )
			$from_name = 'WordPress';

		// Plugin authors can override the potentially troublesome default
		$body["from_name"] = apply_filters( 'wp_mail_from_name', $from_name  );

		// Set mail's subject and body
		$body["subject"] = $subject;
		$body["body"]    = $message;

		// Collect recipients emails into objects to match the TMS template
		foreach($recipients as $recipient) {
			$body["recipients"][] = (object) array(
				"email"      => $recipient
			);
		}

		$headers = array(
			'X-AUTH-TOKEN'   => $auth_token,
			'Content-Type'   => 'application/json',
			'charset'        => 'utf-8'
		);

		$args = array(
			'method' => 'POST',
			'timeout' => 5,
			'redirection' => 5,
			'httpversion' => '1.0',
			'blocking' => true,
			'headers' => $headers,
			'body' => json_encode($body)
		);

		$response = wp_remote_post(
			$tms_url,
			$args
		);

		// Send!
		if ( is_wp_error( $response ) ) {
			//$error_message = $response->get_error_message();
			return false;
		} else {
			return $response;
		}

	}
endif;

