<?php
/**
 * TMS Mailer.
 *
 * @package   TMS_Mailer
 * @author    Spyder Trap
 * @license   GPL-2.0+
 * @link      http://spydertrap.com
 * @copyright 2013 Spyder Trap
 */

/**
 * @package   TMS_Mailer
 * @author    Spyder Trap <admin@spydertrap.com>
 */
class TMS_Mailer {

	/**
	 * Plugin version, used for cache-busting of style and script file references.
	 *
	 * @since   1.0.0
	 *
	 * @var     string
	 */
	const VERSION = '1.0.0';

	/**
	 * Unique identifier for the plugin.
	 *
	 * The variable name is used as the text domain when internationalizing strings
	 * of text. Its value should match the Text Domain file header in the main
	 * plugin file.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected $plugin_slug = 'tms-mailer';

	/**
	 * Instance of this class.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	protected static $instance = null;

	/**
	 * Initialize the plugin by setting localization and loading public scripts
	 * and styles.
	 *
	 * @since 1.0.0
	 */
	private function __construct() {

		// Load plugin text domain
		add_action( 'init', array( $this, 'load_plugin_textdomain' ) );

		// Activate plugin when new blog is added
		add_action( 'wpmu_new_blog', array( $this, 'activate_new_site' ) );

		// Load public-facing style sheet and JavaScript.
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

		// Add custom cron schedules
		add_filter( 'cron_schedules', array( $this, 'add_cron_schedules' ) );

		//Hook the function send_mails_in_queue() into the action tms_cron_job
		add_action( 'tms_cron_job', array( $this, 'send_emails_in_queue' ) );

	}

	/**
	 * Return the plugin slug.
	 *
	 * @since 1.0.0
	 *
	 *@return string Plugin slug variable.
	 */
	public function get_plugin_slug() {
		return $this->plugin_slug;
	}

	/**
	 * Return an instance of this class.
	 *
	 * @since 1.0.0
	 *
	 * @return object    A single instance of this class.
	 */
	public static function get_instance() {

		// If the single instance hasn't been set, set it now.
		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}

	/**
	 * Fired when the plugin is activated.
	 *
	 * @since 1.0.0
	 *
	 * @param    boolean    $network_wide    True if WPMU superadmin uses
	 *                                       "Network Activate" action, false if
	 *                                       WPMU is disabled or plugin is
	 *                                       activated on an individual blog.
	 */
	public static function activate( $network_wide ) {

		if ( function_exists( 'is_multisite' ) && is_multisite() ) {

			if ( $network_wide  ) {

				// Get all blog ids
				$blog_ids = self::get_blog_ids();

				foreach ( $blog_ids as $blog_id ) {

					switch_to_blog( $blog_id );
					self::single_activate();
				}

				restore_current_blog();

			} else {
				self::single_activate();
			}

		} else {
			self::single_activate();
		}

	}

	/**
	 * Fired when the plugin is deactivated.
	 *
	 * @since 1.0.0
	 *
	 * @param    boolean    $network_wide    True if WPMU superadmin uses
	 *                                       "Network Deactivate" action, false if
	 *                                       WPMU is disabled or plugin is
	 *                                       deactivated on an individual blog.
	 */
	public static function deactivate( $network_wide ) {

		if ( function_exists( 'is_multisite' ) && is_multisite() ) {

			if ( $network_wide ) {

				// Get all blog ids
				$blog_ids = self::get_blog_ids();

				foreach ( $blog_ids as $blog_id ) {

					switch_to_blog( $blog_id );
					self::single_deactivate();

				}

				restore_current_blog();

			} else {
				self::single_deactivate();
			}

		} else {
			self::single_deactivate();
		}

	}

	/**
	 * Fired when a new site is activated with a WPMU environment.
	 *
	 * @since 1.0.0
	 *
	 * @param    int    $blog_id    ID of the new blog.
	 */
	public function activate_new_site( $blog_id ) {

		if ( 1 !== did_action( 'wpmu_new_blog' ) ) {
			return;
		}

		switch_to_blog( $blog_id );
		self::single_activate();
		restore_current_blog();

	}

	/**
	 * Get all blog ids of blogs in the current network that are:
	 * - not archived
	 * - not spam
	 * - not deleted
	 *
	 * @since 1.0.0
	 *
	 * @return   array|false    The blog ids, false if no matches.
	 */
	private static function get_blog_ids() {

		global $wpdb;

		// get an array of blog ids
		$sql = "SELECT blog_id FROM $wpdb->blogs
			WHERE archived = '0' AND spam = '0'
			AND deleted = '0'";

		return $wpdb->get_col( $sql );

	}

	/**
	 * Fired for each blog when the plugin is activated.
	 * Adding a table
	 *
	 * @since 1.0.0
	 */
	private static function single_activate() {
		global $wpdb;
		$db_version = "1.0";

		$table_name = $wpdb->prefix . "tms_mails";

		$sql = "CREATE TABLE IF NOT EXISTS $table_name (
  id BIGINT NOT NULL AUTO_INCREMENT,
  email VARCHAR(55) DEFAULT '' NOT NULL,
  subject tinytext NOT NULL,
  body text NOT NULL,
  time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
  UNIQUE KEY id (id)
    );";

		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		dbDelta( $sql );

		add_option( "tms_db_version", $db_version );
	}

	/**
	 * Fired for each blog when the plugin is deactivated.
	 *
	 * @since 1.0.0
	 */
	private static function single_deactivate() {
		$timestamp = wp_next_scheduled( 'tms_cron_job' );
		if ($timestamp !== false ) {
			wp_clear_scheduled_hook( 'tms_cron_job' );
			self::send_emails_before_cron_disable();
		}
	}

	/**
	 * Load the plugin text domain for translation.
	 *
	 * @since 1.0.0
	 */
	public function load_plugin_textdomain() {

		$domain = $this->plugin_slug;
		$locale = apply_filters( 'plugin_locale', get_locale(), $domain );

		load_textdomain( $domain, trailingslashit( WP_LANG_DIR ) . $domain . '/' . $domain . '-' . $locale . '.mo' );

	}

	/**
	 * Register and enqueue public-facing style sheet.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( $this->plugin_slug . '-plugin-styles', plugins_url( 'assets/css/public.css', __FILE__ ), array(), self::VERSION );
	}

	/**
	 * Register and enqueues public-facing JavaScript files.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( $this->plugin_slug . '-plugin-script', plugins_url( 'assets/js/public.js', __FILE__ ), array( 'jquery' ), self::VERSION );
	}

	/**
	 * Check whether rerouting can be enabled
	 *
	 * @since 1.0.0
	 * @return bool
	 */
	public function is_tms_enabled() {
		$options = get_option($this->plugin_slug.'_options');

		if (
				isset($options[$this->plugin_slug.'_rerouting_select']) &&
				isset($options[$this->plugin_slug.'_url']) &&
				isset($options[$this->plugin_slug.'_auth_token']) &&
				$options[$this->plugin_slug.'_rerouting_select'] == '1' &&
				$options[$this->plugin_slug.'_url'] != "" &&
				$options[$this->plugin_slug.'_auth_token'] != ""
		) {
			return true;
		} else if (
				isset($options[$this->plugin_slug.'_rerouting_select']) &&
				$options[$this->plugin_slug.'_rerouting_select'] == '1'
		) {
			return 'need_fields';
		} else {
			return false;
		}
	}

	/**
	 * Check whether cron can be enabled
	 *
	 * @since 1.0.0
	 * @return bool
	 */
	public function is_cron_enabled() {
		$options = get_option($this->plugin_slug.'_options');
		$timestamp = wp_next_scheduled( 'tms_cron_job' );

		if (
				$timestamp !== false &&
				$this->is_tms_enabled() === true &&
				isset($options[$this->plugin_slug.'_cron_select']) &&
				isset($options[$this->plugin_slug.'_queue_quantity']) &&
				$options[$this->plugin_slug.'_cron_select'] == '1' &&
				$options[$this->plugin_slug.'_queue_quantity'] != ""
		) {
			return true;
		} else if (
				$timestamp == false &&
				$this->is_tms_enabled() === true &&
				isset($options[$this->plugin_slug.'_cron_select']) &&
				isset($options[$this->plugin_slug.'_queue_quantity']) &&
				$options[$this->plugin_slug.'_cron_select'] == '1' &&
				$options[$this->plugin_slug.'_queue_quantity'] != ""
		) {
			return 'need_reactivate';
		} else if (
				$this->is_tms_enabled() === true &&
				isset($options['tms-mailer_cron_select']) &&
				$options['tms-mailer_cron_select'] == '1') {
			return 'need_fields';
		} else {
			return false;
		}
	}

	/**
	 * Helper function for registering custom schedules for cron
	 * Defaults are hourly, twicedaily, daily
	 *
	 * @param $schedules
	 *
	 * @since 1.0.0
	 * @return mixed
	 */
	public function add_cron_schedules( $schedules ) {

		$schedules['five_minutes'] = array(
			'interval' => 5 * 60,
			'display' => __( 'Every 5 minutes', $this->plugin_slug )
		);
		$schedules['ten_minutes'] = array(
			'interval' => 10 * 60,
			'display' => __( 'Every 10 minutes', $this->plugin_slug )
		);
		$schedules['fifteen_minutes'] = array(
			'interval' => 15 * 60,
			'display' => __( 'Every 15 minutes', $this->plugin_slug )
		);
		$schedules['thirty_minutes'] = array(
			'interval' => 30 * 60,
			'display' => __( 'Every 30 minutes', $this->plugin_slug )
		);
		$schedules['fourty_five_minutes'] = array(
			'interval' => 45 * 60,
			'display' => __( 'Every 45 minutes', $this->plugin_slug )
		);
		$schedules['two_hours'] = array(
			'interval' => 120 * 60,
			'display' => __( 'Every 2 hours', $this->plugin_slug )
		);
		$schedules['three_hours'] = array(
			'interval' => 180 * 60,
			'display' => __( 'Every 3 hours', $this->plugin_slug )
		);
		$schedules['six_hours'] = array(
			'interval' => 360 * 60,
			'display' => __( 'Every 6 hours', $this->plugin_slug )
		);

		return $schedules;
	}

	/**
	 * Send emails in queue (stored in db)
	 *
	 * @since 1.0.0
	 */
	public function send_emails_in_queue() {
		global $wpdb;

		$ids = array();

		// get plugins options
		$options = get_option($this->plugin_slug.'_options');

		// set table name and get rows from db
		$table_name = $wpdb->prefix . "tms_mails";
		$rows = $wpdb->get_results( "SELECT * FROM $table_name" );

		// count rows and get quantity from settings
		$queue = $options[$this->plugin_slug.'_queue_quantity'];
		$rows_count = count($rows);

		// if no rows were found
		if ($rows_count == 0)
			return;

		// check whether number of rows is less that the queue number
		$count = $rows_count < (int) $queue ? $rows_count : (int) $queue;

		// send only emails in queue
		for($i = 0; $i < $count; $i++) {
			wp_mail($rows[$i]->email, $rows[$i]->subject, $rows[$i]->body, false, false, true);

			// set sent email ids to the array
			$ids[] = (int) $rows[$i]->id;
		}

		// delete sent emails from db
		$sql = $wpdb->prepare("DELETE FROM $table_name WHERE id IN (". str_repeat("%d,", count($ids)-1) . "%d)" , $ids);
		$wpdb->query($sql);
	}

	/**
	 * Send emails before disabling cron or deactivating a plugin
	 *
	 * @since 1.0.0
	 */
	public function send_emails_before_cron_disable() {
		global $wpdb;
		$ids = array();

		// set table name and get rows from db
		$table_name = $wpdb->prefix . "tms_mails";
		$rows = $wpdb->get_results( "SELECT * FROM $table_name" );

		// count rows
		$rows_count = count($rows);

		// if no rows were found
		if ($rows_count == 0)
			return;

		// send all emails in db
		for($i = 0; $i < $rows_count; $i++) {
			wp_mail($rows[$i]->email, $rows[$i]->subject, $rows[$i]->body, false, false, true);

			// set sent email ids to the array
			$ids[] = (int) $rows[$i]->id;
		}

		// delete sent emails from db
		$sql = $wpdb->prepare("DELETE FROM $table_name WHERE id IN (". str_repeat("%d,", count($ids)-1) . "%d)" , $ids);
		$wpdb->query($sql);
	}
}